@@ -4,17 +4,19 @@ import unittest
 
 from pycps import downloaders as d
 
+curdir = os.path.dirname(__file__)
+
 
 class TestDownloaders(unittest.TestCase):
 
     def test_all_monthly_files(self):
-        result = list(d.all_monthly_files('files/trimmed_nber.html'))
+        result = list(d.all_monthly_files(curdir + '/files/trimmed_nber.html'))
         expected = [('cpsb7601.Z', 'cpsm1976-01.Z'),
                     ('jan94pub.zip', 'cpsm1994-01.zip')]
         self.assertEqual(result, expected)
 
     def test_all_monthly_dd(self):
-        result = list(d.all_monthly_files('files/trimmed_nber.html',
+        result = list(d.all_monthly_files(curdir + '/files/trimmed_nber.html',
                                           kind='dictionary'))
         expected = [('cpsbjan03.ddf', 'cpsm2003-01.ddf')]
         self.assertEqual(result, expected)
@@ -54,7 +56,7 @@ class TestDownloaders(unittest.TestCase):
                  ('apr94.zip', 'cpsm1994-04.zip'),
                  ('may94.zip', 'cpsm1994-05.zip')]
         months = ['1994-01', '1994-03', '1994-04']
-        result = list(d.filter_monthly_files(files, months=months))
+        result = list(d.filter_monthly(files, months=months))
         expected = files[0:3]
         self.assertEqual(result, expected)
 
@@ -66,7 +68,7 @@ class TestDownloaders(unittest.TestCase):
                  ('dec94.zip', 'cpsm1994-12.zip'),
                  ('jan95.zip', 'cpsm1995-01.zip')]
         months = [['1994-01', '1994-04'], ['1994-11', '1995-01']]
-        result = list(d.filter_monthly_files(files, months=months))
+        result = list(d.filter_monthly(files, months=months))
         expected = files
         self.assertEqual(result, expected)
 
